@@ -1,7 +1,7 @@
 #!/bin/sh
 
-# Copyright (c) 2012 - 2014 dak180 and contributors. See
-# http://opensource.org/licenses/mit-license.php or the included
+# Copyright (c) 2012 - 2016 dak180 and contributors. See
+# https://opensource.org/licenses/mit-license.php or the included
 # COPYING.md for licence terms.
 #
 # autorevision - extracts metadata about the head version from your
@@ -22,6 +22,7 @@ usage: autorevision {-t output-type | -s symbol} [-o cache-file [-f] ] [-V]
 
 The following are valid output types:
 	clojure			= clojure file
+	c			= C/C++ file
 	h			= Header for use with c/c++
 	hpp			= Alternate C++ header strings with namespace
 	ini			= INI file
@@ -57,6 +58,7 @@ The following are valid symbols:
 	VCS_FULL_HASH
 	VCS_SHORT_HASH
 	VCS_WC_MODIFIED
+	VCS_ACTION_STAMP
 EOF
 	exit 1
 }
@@ -117,8 +119,10 @@ fi
 
 # Functions to extract data from different repo types.
 # For git repos
-# shellcheck disable=SC2155
+# shellcheck disable=SC2039,SC2164,SC2155
 gitRepo() {
+	local oldPath="${PWD}"
+
 	cd "$(git rev-parse --show-toplevel)"
 
 	VCS_TYPE="git"
@@ -171,11 +175,28 @@ gitRepo() {
 	fi
 
 	# Date of the current commit
-	VCS_DATE="$(git log -1 --pretty=format:%ci | sed -e 's: :T:' -e 's: ::')"
+	VCS_DATE="$(TZ=UTC git show -s --date=iso-strict-local --pretty=format:%ad | sed -e 's|+00:00|Z|')"
+	if [ -z "${VCS_DATE}" ]; then
+		echo "warning: Action stamps require git version 2.7+." 1>&2
+		VCS_DATE="$(git log -1 --pretty=format:%ci | sed -e 's: :T:' -e 's: ::' -e 's|+00:00|Z|')"
+		local ASdis="1"
+	fi
+
+	# Action Stamp
+	if [ -z "${ASdis}" ]; then
+		VCS_ACTION_STAMP="${VCS_DATE}!$(git show -s --pretty=format:%cE)"
+	else
+		VCS_ACTION_STAMP=""
+	fi
+
+	cd "${oldPath}"
 }
 
 # For hg repos
+# shellcheck disable=SC2039,SC2164
 hgRepo() {
+	local oldPath="${PWD}"
+
 	cd "$(hg root)"
 
 	VCS_TYPE="hg"
@@ -206,7 +227,7 @@ hgRepo() {
 	fi
 
 	# Current or last tag ancestor (excluding auto tags, empty if no tags)
-	VCS_TAG="$(hg log -r "${VCS_NUM}" -l 1 --template '{latesttag}\n' 2>/dev/null | sed -e 's:qtip::' -e 's:tip::' -e 's:qbase::' -e 's:qparent::' -e "s:$(hg --config 'extensions.color=' --color never qtop 2>/dev/null)::" | cut -d ' ' -f 1)"
+	VCS_TAG="$(hg log -r "${VCS_NUM}" -l 1 --template '{latesttag}\n' 2>/dev/null | sed -e 's:qtip::' -e 's:tip::' -e 's:qbase::' -e 's:qparent::' -e "s:$(hg --config 'extensions.color=' --config 'extensions.mq=' --color never qtop 2>/dev/null)::" | cut -d ' ' -f 1)"
 
 	# Distance to last tag or an alias of VCS_NUM if there is no tag
 	if [ ! -z "${VCS_TAG}" ]; then
@@ -216,11 +237,19 @@ hgRepo() {
 	fi
 
 	# Date of the current commit
-	VCS_DATE="$(hg log -r "${VCS_NUM}" -l 1 --template '{date|isodatesec}\n' 2>/dev/null | sed -e 's: :T:' -e 's: ::')"
+	VCS_DATE="$(hg log -r "${VCS_NUM}" -l 1 --template '{date|isodatesec}\n' 2>/dev/null | sed -e 's: :T:' -e 's: ::' -e 's|+00:00|Z|')"
+
+	# Action Stamp
+	VCS_ACTION_STAMP="$(TZ=UTC hg log -r "${VCS_NUM}" -l 1 --template '{date|localdate|rfc3339date}\n' 2>/dev/null | sed -e 's|+00:00|Z|')!$(hg log -r "${VCS_NUM}" -l 1 --template '{author|email}\n' 2>/dev/null)"
+
+	cd "${oldPath}"
 }
 
 # For bzr repos
+# shellcheck disable=SC2039,SC2164
 bzrRepo() {
+	local oldPath="${PWD}"
+
 	cd "$(bzr root)"
 
 	VCS_TYPE="bzr"
@@ -258,11 +287,19 @@ bzrRepo() {
 
 	# Date of the current commit
 	VCS_DATE="$(bzr version-info --custom --template='{date}\n' | sed -e 's: :T:' -e 's: ::')"
+
+	# Action Stamp
+	# Currently unimplemented because more investigation is needed.
+	VCS_ACTION_STAMP=""
+
+	cd "${oldPath}"
 }
 
 # For svn repos
-# shellcheck disable=SC2155
+# shellcheck disable=SC2039,SC2164,SC2155
 svnRepo() {
+	local oldPath="${PWD}"
+
 	VCS_TYPE="svn"
 
 	case "${PWD}" in
@@ -347,11 +384,66 @@ svnRepo() {
 	VCS_TICK=""
 
 	# Date of the current commit
-	VCS_DATE="$(svn info --xml | sed -n -e 's:<date>::' -e 's:</date>::' -e 's:Z:-0000:p')"
+	VCS_DATE="$(svn info --xml | sed -n -e 's:<date>::' -e 's:</date>::p')"
+
+	# Action Stamp
+	VCS_ACTION_STAMP="${VCS_DATE}!$(svn log --xml -l 1 -r "${VCS_SHORT_HASH}" | sed -n -e 's:<author>::' -e 's:</author>::p')"
+
+	cd "${oldPath}"
 }
 
 
 # Functions to output data in different formats.
+# For bash output
+shOutput() {
+	cat > "${TARGETFILE}" << EOF
+# Generated by autorevision - do not hand-hack!
+
+VCS_TYPE="${VCS_TYPE}"
+VCS_BASENAME="${VCS_BASENAME}"
+VCS_UUID="${VCS_UUID}"
+VCS_NUM="${VCS_NUM}"
+VCS_DATE="${VCS_DATE}"
+VCS_BRANCH="${VCS_BRANCH}"
+VCS_TAG="${VCS_TAG}"
+VCS_TICK="${VCS_TICK}"
+VCS_EXTRA="${VCS_EXTRA}"
+
+VCS_ACTION_STAMP="${VCS_ACTION_STAMP}"
+VCS_FULL_HASH="${VCS_FULL_HASH}"
+VCS_SHORT_HASH="${VCS_SHORT_HASH}"
+
+VCS_WC_MODIFIED="${VCS_WC_MODIFIED}"
+
+# end
+EOF
+}
+
+# For source C output
+cOutput() {
+	cat > "${TARGETFILE}" << EOF
+/* Generated by autorevision - do not hand-hack! */
+
+const char *VCS_TYPE         = "${VCS_TYPE}";
+const char *VCS_BASENAME     = "${VCS_BASENAME}";
+const char *VCS_UUID         = "${VCS_UUID}";
+const int VCS_NUM            = ${VCS_NUM};
+const char *VCS_DATE         = "${VCS_DATE}";
+const char *VCS_BRANCH       = "${VCS_BRANCH}";
+const char *VCS_TAG          = "${VCS_TAG}";
+const int VCS_TICK           = ${VCS_TICK};
+const char *VCS_EXTRA        = "${VCS_EXTRA}";
+
+const char *VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}";
+const char *VCS_FULL_HASH    = "${VCS_FULL_HASH}";
+const char *VCS_SHORT_HASH   = "${VCS_SHORT_HASH}";
+
+const int VCS_WC_MODIFIED     = ${VCS_WC_MODIFIED};
+
+/* end */
+EOF
+}
+
 # For header output
 hOutput() {
 	cat > "${TARGETFILE}" << EOF
@@ -369,6 +461,7 @@ hOutput() {
 #define VCS_TICK		${VCS_TICK}
 #define VCS_EXTRA		"${VCS_EXTRA}"
 
+#define VCS_ACTION_STAMP	"${VCS_ACTION_STAMP}"
 #define VCS_FULL_HASH		"${VCS_FULL_HASH}"
 #define VCS_SHORT_HASH		"${VCS_SHORT_HASH}"
 
@@ -397,6 +490,7 @@ xcodeOutput() {
 #define VCS_TICK		${VCS_TICK}
 #define VCS_EXTRA		${VCS_EXTRA}
 
+#define VCS_ACTION_STAMP	${VCS_ACTION_STAMP}
 #define VCS_FULL_HASH		${VCS_FULL_HASH}
 #define VCS_SHORT_HASH		${VCS_SHORT_HASH}
 
@@ -432,6 +526,11 @@ swiftOutput() {
 	else
 		VCS_EXTRA="\"${VCS_EXTRA}\""
 	fi
+	if [ -z "${VCS_ACTION_STAMP}" ]; then
+		VCS_ACTION_STAMP="nil"
+	else
+		VCS_ACTION_STAMP="\"${VCS_ACTION_STAMP}\""
+	fi
 	cat > "${TARGETFILE}" << EOF
 /* Generated by autorevision - do not hand-hack! */
 
@@ -445,6 +544,7 @@ let VCS_TAG:	String?	= ${VCS_TAG}
 let VCS_TICK:	Int?	= ${VCS_TICK}
 let VCS_EXTRA:	String?	= ${VCS_EXTRA}
 
+let VCS_ACTION_STAMP:	String?	= ${VCS_ACTION_STAMP}
 let VCS_FULL_HASH:		String	= "${VCS_FULL_HASH}"
 let VCS_SHORT_HASH:		String	= "${VCS_SHORT_HASH}"
 
@@ -454,30 +554,6 @@ let VCS_WC_MODIFIED:	Bool	= ${VCS_WC_MODIFIED}
 EOF
 }
 
-# For bash output
-shOutput() {
-	cat > "${TARGETFILE}" << EOF
-# Generated by autorevision - do not hand-hack!
-
-VCS_TYPE="${VCS_TYPE}"
-VCS_BASENAME="${VCS_BASENAME}"
-VCS_UUID="${VCS_UUID}"
-VCS_NUM="${VCS_NUM}"
-VCS_DATE="${VCS_DATE}"
-VCS_BRANCH="${VCS_BRANCH}"
-VCS_TAG="${VCS_TAG}"
-VCS_TICK="${VCS_TICK}"
-VCS_EXTRA="${VCS_EXTRA}"
-
-VCS_FULL_HASH="${VCS_FULL_HASH}"
-VCS_SHORT_HASH="${VCS_SHORT_HASH}"
-
-VCS_WC_MODIFIED="${VCS_WC_MODIFIED}"
-
-# end
-EOF
-}
-
 # For Python output
 pyOutput() {
 	case "${VCS_WC_MODIFIED}" in
@@ -497,6 +573,7 @@ VCS_TAG = "${VCS_TAG}"
 VCS_TICK = ${VCS_TICK}
 VCS_EXTRA = "${VCS_EXTRA}"
 
+VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}"
 VCS_FULL_HASH = "${VCS_FULL_HASH}"
 VCS_SHORT_HASH = "${VCS_SHORT_HASH}"
 
@@ -521,6 +598,7 @@ plOutput() {
 \$VCS_TICK = ${VCS_TICK};
 \$VCS_EXTRA = "${VCS_EXTRA}";
 
+\$VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}";
 \$VCS_FULL_HASH = "${VCS_FULL_HASH}";
 \$VCS_SHORT_HASH = "${VCS_SHORT_HASH}";
 
@@ -549,6 +627,7 @@ VCS_TAG = "${VCS_TAG}"
 VCS_TICK = ${VCS_TICK}
 VCS_EXTRA = "${VCS_EXTRA}"
 
+VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}"
 VCS_FULL_HASH = "${VCS_FULL_HASH}"
 VCS_SHORT_HASH = "${VCS_SHORT_HASH}"
 
@@ -568,20 +647,21 @@ phpOutput() {
 <?php
 # Generated by autorevision - do not hand-hack!
 
-$GLOBALS["VCS_TYPE"] = "${VCS_TYPE}";
-$GLOBALS["VCS_BASENAME"] = "${VCS_BASENAME}";
-$GLOBALS["VCS_UUID"] = "${VCS_UUID}";
-$GLOBALS["VCS_NUM"] = ${VCS_NUM};
-$GLOBALS["VCS_DATE"] = "${VCS_DATE}";
-$GLOBALS["VCS_BRANCH"] = "${VCS_BRANCH}";
-$GLOBALS["VCS_TAG"] = "${VCS_TAG}";
-$GLOBALS["VCS_TICK"] = ${VCS_TICK};
-$GLOBALS["VCS_EXTRA"] = "${VCS_EXTRA}";
-
-$GLOBALS["VCS_FULL_HASH"] = "${VCS_FULL_HASH}";
-$GLOBALS["VCS_SHORT_HASH"] = "${VCS_SHORT_HASH}";
-
-$GLOBALS["VCS_WC_MODIFIED"] = ${VCS_WC_MODIFIED};
+return array(
+	"VCS_TYPE" => "${VCS_TYPE}",
+	"VCS_BASENAME" => "${VCS_BASENAME}",
+	"VCS_UUID" => "${VCS_UUID}",
+	"VCS_NUM" => ${VCS_NUM},
+	"VCS_DATE" => "${VCS_DATE}",
+	"VCS_BRANCH" => "${VCS_BRANCH}",
+	"VCS_TAG" => "${VCS_TAG}",
+	"VCS_TICK" => ${VCS_TICK},
+	"VCS_EXTRA" => "${VCS_EXTRA}",
+	"VCS_ACTION_STAMP" => "${VCS_ACTION_STAMP}",
+	"VCS_FULL_HASH" => "${VCS_FULL_HASH}",
+	"VCS_SHORT_HASH" => "${VCS_SHORT_HASH}",
+	"VCS_WC_MODIFIED" => ${VCS_WC_MODIFIED}
+);
 
 # end
 ?>
@@ -606,6 +686,7 @@ VCS_BRANCH = "${VCS_BRANCH}"
 VCS_TAG = "${VCS_TAG}"
 VCS_TICK = ${VCS_TICK}
 VCS_EXTRA = "${VCS_EXTRA}"
+VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}"
 VCS_FULL_HASH = "${VCS_FULL_HASH}"
 VCS_SHORT_HASH = "${VCS_SHORT_HASH}"
 VCS_WC_MODIFIED = ${VCS_WC_MODIFIED}
@@ -633,6 +714,7 @@ var autorevision = {
 	VCS_TICK: ${VCS_TICK},
 	VCS_EXTRA: "${VCS_EXTRA}",
 
+	VCS_ACTION_STAMP: "${VCS_ACTION_STAMP}",
 	VCS_FULL_HASH: "${VCS_FULL_HASH}",
 	VCS_SHORT_HASH: "${VCS_SHORT_HASH}",
 
@@ -666,6 +748,7 @@ jsonOutput() {
 	"VCS_TICK": ${VCS_TICK},
 	"VCS_EXTRA": "${VCS_EXTRA}",
 
+	"VCS_ACTION_STAMP": "${VCS_ACTION_STAMP}",
 	"VCS_FULL_HASH": "${VCS_FULL_HASH}",
 	"VCS_SHORT_HASH": "${VCS_SHORT_HASH}",
 
@@ -694,6 +777,7 @@ public class autorevision {
     public static final long VCS_TICK = ${VCS_TICK};
     public static final String VCS_EXTRA = "${VCS_EXTRA}";
 
+    public static final String VCS_ACTION_STAMP = "${VCS_ACTION_STAMP}";
     public static final String VCS_FULL_HASH = "${VCS_FULL_HASH}";
     public static final String VCS_SHORT_HASH = "${VCS_SHORT_HASH}";
 
@@ -721,6 +805,7 @@ VCS_TAG=${VCS_TAG}
 VCS_TICK=${VCS_TICK}
 VCS_EXTRA=${VCS_EXTRA}
 
+VCS_ACTION_STAMP=${VCS_ACTION_STAMP}
 VCS_FULL_HASH=${VCS_FULL_HASH}
 VCS_SHORT_HASH=${VCS_SHORT_HASH}
 
@@ -740,6 +825,7 @@ define(\`VCS_BRANCH', \`${VCS_BRANCH}')dnl
 define(\`VCS_TAG', \`${VCS_TAG}')dnl
 define(\`VCS_TICK', \`${VCS_TICK}')dnl
 define(\`VCS_EXTRA', \`${VCS_EXTRA}')dnl
+define(\`VCS_ACTIONSTAMP', \`${VCS_ACTION_STAMP}')dnl
 define(\`VCS_FULLHASH', \`${VCS_FULL_HASH}')dnl
 define(\`VCS_SHORTHASH', \`${VCS_SHORT_HASH}')dnl
 define(\`VCS_WC_MODIFIED', \`${VCS_WC_MODIFIED}')dnl
@@ -763,6 +849,7 @@ texOutput() {
 \def \vcsTag {${VCS_TAG}}
 \def \vcsTick {${VCS_TICK}}
 \def \vcsExtra {${VCS_EXTRA}}
+\def \vcsACTIONSTAMP {${VCS_ACTION_STAMP}}
 \def \vcsFullHash {${VCS_FULL_HASH}}
 \def \vcsShortHash {${VCS_SHORT_HASH}}
 \def \vcsWCModified {${VCS_WC_MODIFIED}}
@@ -788,6 +875,7 @@ schemeOutput() {
 (define VCS_TICK        ${VCS_TICK})
 (define VCS_EXTRA       "${VCS_EXTRA}")
 
+(define VCS_ACTION_STAMP   "${VCS_ACTION_STAMP}")
 (define VCS_FULL_HASH   "${VCS_FULL_HASH}")
 (define VCS_SHORT_HASH  "${VCS_SHORT_HASH}")
 
@@ -814,8 +902,9 @@ clojureOutput() {
 (def VCS_TICK        ${VCS_TICK})
 (def VCS_EXTRA       "${VCS_EXTRA}")
 
-(def VCS_FULL_HASH   "${VCS_FULL_HASH}")
-(def VCS_SHORT_HASH  "${VCS_SHORT_HASH}")
+(def VCS_ACTION_STAMP   "${VCS_ACTION_STAMP}")
+(def VCS_FULL_HASH      "${VCS_FULL_HASH}")
+(def VCS_SHORT_HASH     "${VCS_SHORT_HASH}")
 
 (def VCS_WC_MODIFIED ${VCS_WC_MODIFIED})
 ;; end
@@ -836,6 +925,7 @@ $([ "${VCS_TAG}" ] && echo "%define vcs_tag			${VCS_TAG}")
 $([ "${VCS_TICK}" ] && echo "%define vcs_tick		${VCS_TICK}")
 $([ "${VCS_EXTRA}" ] && echo "%define vcs_extra		${VCS_EXTRA}")
 
+$([ "${VCS_ACTION_STAMP}" ] && echo "%define vcs_action_stamp		${VCS_ACTION_STAMP}")
 $([ "${VCS_FULL_HASH}" ] && echo "%define vcs_full_hash		${VCS_FULL_HASH}")
 $([ "${VCS_SHORT_HASH}" ] && echo "%define vcs_short_hash		${VCS_SHORT_HASH}")
 
@@ -844,7 +934,7 @@ $([ "${VCS_WC_MODIFIED}" ] && echo "%define vcs_wc_modified		${VCS_WC_MODIFIED}"
 EOF
 }
 
-# shellcheck disable=SC2155
+# shellcheck disable=SC2155,SC2039
 hppOutput() {
 	local NAMESPACE="$(echo "${VCS_BASENAME}" | sed -e 's:_::g' | tr '[:lower:]' '[:upper:]')"
 	cat > "${TARGETFILE}" << EOF
@@ -865,6 +955,7 @@ namespace $(echo "${NAMESPACE}" | tr '[:upper:]' '[:lower:]')
 	const int VCS_TICK				= ${VCS_TICK};
 	const std::string VCS_EXTRA		= "${VCS_EXTRA}";
 
+	const std::string VCS_ACTION_STAMP	= "${VCS_ACTION_STAMP}";
 	const std::string VCS_FULL_HASH		= "${VCS_FULL_HASH}";
 	const std::string VCS_SHORT_HASH	= "${VCS_SHORT_HASH}";
 
@@ -895,6 +986,7 @@ VCS_TAG = '${VCS_TAG}';
 VCS_TICK = ${VCS_TICK};
 VCS_EXTRA = '${VCS_EXTRA}';
 
+VCS_ACTION_STAMP = '${VCS_ACTION_STAMP}';
 VCS_FULL_HASH = '${VCS_FULL_HASH}';
 VCS_SHORT_HASH = '${VCS_SHORT_HASH}';
 
@@ -918,6 +1010,7 @@ VCS_TAG = '${VCS_TAG}';
 VCS_TICK = ${VCS_TICK};
 VCS_EXTRA = '${VCS_EXTRA}';
 
+VCS_ACTION_STAMP = '${VCS_ACTION_STAMP}';
 VCS_FULL_HASH = '${VCS_FULL_HASH}';
 VCS_SHORT_HASH = '${VCS_SHORT_HASH}';
 
@@ -930,12 +1023,14 @@ EOF
 
 # Helper functions
 # Count path segments
+# shellcheck disable=SC2039
 pathSegment() {
 	local pathz="${1}"
 	local depth="0"
 
 	if [ ! -z "${pathz}" ]; then
-		while [ ! "${pathz}" = "/" ] && [ ! "${pathz}" = "." ]; do
+		# Continue until we are at / or there are no path separators left.
+		while [ ! "${pathz}" = "/" ] && [ ! "${pathz}" = "$(echo "${pathz}" | sed -e 's:/::')" ]; do
 			pathz="$(dirname "${pathz}")"
 			depth="$((depth+1))"
 		done
@@ -944,6 +1039,7 @@ pathSegment() {
 }
 
 # Largest of four numbers
+# shellcheck disable=SC2039
 multiCompare() {
 	local larger="${1}"
 	local numA="${2}"
@@ -956,8 +1052,8 @@ multiCompare() {
 	echo "${larger}"
 }
 
-# shellcheck disable=SC2155
 # Test for repositories
+# shellcheck disable=SC2155,SC2039
 repoTest() {
 	REPONUM="0"
 	if [ ! -z "$(git rev-parse HEAD 2>/dev/null)" ]; then
@@ -1018,6 +1114,7 @@ repoTest() {
 
 
 # Detect which repos we are in and gather data.
+# shellcheck source=/dev/null
 if [ -f "${CACHEFILE}" ] && [ "${CACHEFORCE}" = "1" ]; then
 	# When requested only read from the cache to populate our symbols.
 	. "${CACHEFILE}"
@@ -1060,6 +1157,8 @@ if [ ! -z "${VAROUT}" ]; then
 		echo "${VCS_SHORT_HASH}"
 	elif [ "${VAROUT}" = "VCS_WC_MODIFIED" ]; then
 		echo "${VCS_WC_MODIFIED}"
+	elif [ "${VAROUT}" = "VCS_ACTION_STAMP" ]; then
+		echo "${VCS_ACTION_STAMP}"
 	else
 		echo "error: Not a valid output symbol." 1>&2
 		exit 1
@@ -1069,7 +1168,9 @@ fi
 
 # Detect requested output type and use it.
 if [ ! -z "${AFILETYPE}" ]; then
-	if [ "${AFILETYPE}" = "h" ]; then
+	if [ "${AFILETYPE}" = "c" ]; then
+		cOutput
+	elif [ "${AFILETYPE}" = "h" ]; then
 		hOutput
 	elif [ "${AFILETYPE}" = "xcode" ]; then
 		xcodeOutput
